
#include "general.h"

// The actions in these async function controls are automatically governed by
// this window's event queue to avoid race conditions. The callbacks returned
// here can be safely called from outside this window.

function oneShotAsyncRaw() {
	var q = [],
	run = function() {
		spawnRaw(function() {
			var z = q;
			q = null;
			if(z !== null) {
				while(z.length > 0) {
					spawnRaw(z.shift());
				}
			}
			// Else trigger already occurred; do nothing
		});
	},
	add = function(fn) {
		requireFunction(fn);
		spawnRaw(function() {
			if(q === null) {
				// Trigger already occurred.
				// Run directly.
				fn();
			}
			else {
				// Queue.
				q.push(fn);
			}
			return;
		});
	};
	run["add"] = add;
	return run;		
}

// For this function, the returned generator function must be called from
// within this thread's context to avoid race conditions. However, the ticket
// functions returned by the generator can be called safely from any context.

function pendingCallAsyncRaw(fn) {
	requireFunction(fn);

	var
	unused = {},
	nextIndex = 0,
	release,
	gen = function(transitionToActive) {
		// Already active--cannot generate new tickets
		if(release === null)
			return null;
		// Going active; release the held ticket
		if(transitionToActive) {
			var z = release;
			release = null;
			return z();
		}
		// Add a ticket
		var ticketIndex = nextIndex++;
		unused[ticketIndex] = true;
		return function() {
			spawnRaw(function() {
				if(!(unused && (ticketIndex in unused)))
					return; // Ticket already used
				// Use ticket
				delete unused[ticketIndex];
				// If unused tickets still exist, don't trigger.
				// Index-semblant keys are tickets.
				for(var key in unused) {
					if((""+AsUInt(key)) === key)
						return false;
				}
				// No tickets are left
				unused = null;
				spawnRaw(fn);
				return;
			});
			return;
		};
	};
	release = gen();
	return gen;
}
