
#ifndef GENERAL_H
#define GENERAL_H

#include "settings.h"

#if 0
#define LOG(args...) console.log(args)
#else
#define LOG(args...)
#endif

// Some lexical abbreviations defined in general.js
#define Error __ERROR
#define TypeError __TYPE_ERROR
#define setTimeout __SET_TIMEOUT
#define document __DOCUMENT
#define window __WINDOW
#define Object __OBJECT
#define Array __ARRAY
#define String __STRING
#define Number __NUMBER
#define Boolean __BOOLEAN
#define null __NULL
#define undefined __UNDEFINED

#if 0
// A large part of the following depends on general.js being included in the
// scope of use.
//
// These macros are capitalized as a reminder that they're not really functions
// and carry all of the dangers of a macro.
#endif

#define Undefined __UNDEFINED
#define Undef Undefined
#define Inf Number.POSITIVE_INFINITY
#define NInf Number.NEGATIVE_INFINITY

#define Args arguments

#define Owns(o,k) (__OBJECT_OWNS.call((o),(k)))
#define Has(o,k) ((k)in(o))

#define GetObjectString(z) (__OBJECT_TO_STRING.call((z)))
#define ObjectTypeIs(type, z) (GetObjectString(z) === ("[object " + (type) + "]"))
#define NativeTypeIs(type, z) (typeof((z)) === (type))
#define CastsToOwnIdentity(fn, z) ((fn)(z) === (z))

#define AsInt(z) ((z)>>0)
#define AsUInt(z) ((z)>>>0)

#define IsUndef(z) ((z)===Undef)
#define IsDef(z) ((z)!==Undef)
#define IsUndefOrNull(z) (IsUndef(z)||((z)===null))
#define IsNothing(z) IsUndefOrNull(z)
#define IsDefNotNull(z) (!IsUndefOrNull(z))
#define IsSomething(z) IsDefNotNull(z)
#define IsInt(z) ((z)===AsInt(z))
#define IsUInt(z) ((z)===AsUInt(z))

#define IsArray(z) ObjectTypeIs("Array", z)
#define IsObject(z) CastsToOwnIdentity(Object, z)
#define IsString(z) CastsToOwnIdentity(String, z)
#define IsNumber(z) CastsToOwnIdentity(Number, z)
#define IsBoolean(z) CastsToOwnIdentity(Boolean, z)
#define IsRegExp(z) ObjectTypeIs("RegExp", z)
#define IsFunction(z) ObjectTypeIs("Function", z)

#define IsListLike(z) (IsObject(z) && IsUInt((z).length))

// Abbreviates a for loop intended to traverse an array/list
#define TraverseWith(counter, start, end) \
	for((counter)=(start); (counter)<(end); ++(counter))

// Same, except it assumes that counter is i, start is 0, and end is l.
#define Traverse TraverseWith(i,0,l)

// Generic list operations
#define Concat(list,args...) (__ARRAY_CONCAT.call(list,##args))
#define ConcatList(list,args) (__ARRAY_CONCAT.apply(list,args))
#define Join(list,args...) (__ARRAY_JOIN.call(list,##args))
#define JoinList(list,args) (__ARRAY_JOIN.apply(list,args))
#define Pop(list,args...) (__ARRAY_POP.call(list,##args))
#define PopList(list,args) (__ARRAY_POP.apply(list,args))
#define Push(list,args...) (__ARRAY_PUSH.call(list,##args))
#define PushList(list,args) (__ARRAY_PUSH.apply(list,args))
#define Reverse(list,args...) (__ARRAY_REVERSE.call(list,##args))
#define ReverseList(list,args) (__ARRAY_REVERSE.apply(list,args))
#define Shift(list,args...) (__ARRAY_SHIFT.call(list,##args))
#define ShiftList(list,args) (__ARRAY_SHIFT.apply(list,args))
#define Slice(list,args...) (__ARRAY_SLICE.call(list,##args))
#define SliceList(list,args) (__ARRAY_SLICE.apply(list,args))
#define Sort(list,args...) (__ARRAY_SORT.call(list,##args))
#define SortList(list,args) (__ARRAY_SORT.apply(list,args))
#define Splice(list,args...) (__ARRAY_SPLICE.call(list,##args))
#define SpliceList(list,args) (__ARRAY_SPLICE.apply(list,args))
#define Unshift(list,args...) (__ARRAY_UNSHIFT.call(list,##args))
#define UnshiftList(list,args) (__ARRAY_UNSHIFT.apply(list,args))

// String substitutes
#define $onload "onload"
#define $onrs "onreadystatechange"
#define $loading "loading"
#define $loaded "loaded"
#define $complete "complete"
#define $createElement "createElement"
#define $script "script"
#define $rs "readyState"


#endif
