
// The script in this file alone (mini-boot.js) is held by its would-be
// copyright holders to be in the public domain. This statement does not extend
// to external files include or are included by this file.

// This script serves as a quick setup and bootstrapping routine for modu.
// It's specifically made to optimize well when compiled, load quickly, and get
// out of the way. It adds a script element in order to load an external file,
// but (unlike the library it's loading) it doesn't wait on the result.
// Instead, it hangs a callback onto the script element's DOM object. It is
// then the loaded script's job to find the callback (e.g. by traversing
// getElementsByTagName('script') for the first $onceKey property), remove the
// callback from the node, and then execute it, accepting the return value as
// parameters.

// Optimized with closure-compiler, this script reduces to c. 300 characters.
// The configuration remains readable and editable at the end of the minified
// form.

#include "settings.h"

(function(params) {
	var
		$script = 'script',
		$head = 'head',
		$type = 'type',
		$src = 'src',
		$home = 'home',
		$createElement = 'createElement',
		$insertBefore = 'insertBefore',
		$getElementsByTagName = 'getElementsByTagName',
		$firstChild = 'firstChild',
		$textJavascript = 'text/javascript',
		$onceKey = ONCE_SETUP_KEY,
		st = setTimeout,
		doc = document,
		ele = doc[$createElement]($script),
		readyCheckDelay = READY_CHECK_DELAY,
		addWhenReady = function() {
			var head = (doc[$head] || doc[$getElementsByTagName]($head))[0];
			head ?
				head[$insertBefore](ele,head[$firstChild]) :
				st(addWhenReady,readyCheckDelay);
		};
	ele[$type] = $textJavascript;
	ele[$src] = params[$home] + (MAIN_SCRIPT_SUBPATH);
	ele[$onceKey] = function() {
		return params;
	};
	st(addWhenReady,0);
})(
	{
		"home": (HOME_PATH)
	}
);
