
#include "general.h"

#define STR(z) (""+(z))

var /** @const */ trueList = [true];

// Casts list elements to strings, then outputs the distinct copies, favoring
// the earliest instance.
function stringUniq(list) {
	if(!isListLike(list))
		rethrow(TypeError());
	var i, l = list.length, buf = {}, out = [], z;
	Traverse {
		z = STR(list[i]);
		if(!owns(buf,z)) {
			out.push(z);
			buf[z] = true;
		}
	}
	return out;
}

function getLoadFunction() {
	return function() {
		return addScript.apply(this,arguments);
	};
}

// fn must be a callback function(key) that accepts a module name as a string
// and returns an object with optional properties r and s. If r is present, the
// value must be list-like and contain zero or more strings that name other
// modules upon which this one depends. If r is absent, [] is assumed. If s is
// present, the value is treated as a location to be loaded as a script. If
// absent, this module is dependencies-only. If the module does not exist, the
// function should return null.
function createModuleLoaderFromFunction(fn) {
	requireFunction(fn);
	function getModule(name) {
		var obj = fn(name), r, s;
		if(!isObject(obj))
			return null;
		r = obj["r"];
		s = obj["s"];
		r = r ? STR(r) :
			isString(r) ? r : null;
		s = isListLike(s) ? stringUniq(s) : [];
		return {"r":r,"s":s}
	}
	
	var referencedModules = {};
	
	function requestModule() {
		spawnApply(requestModuleBasis, arguments);
	}
	
	function requestModuleBasis(name, onDone) {
		name = STR(name);
		if(owns(referencedModules,name)) {
			// Already requested; queue the callback.
			isFunction(onDone) && referencedModules[name](onDone);
			return;
		}
		var mod = getModule(STR(name)),
			addHandler, trigger, // One-shot used for dependent calls
			allDone, // Triggers one-shot when requirements and script are done
			requires, script,
			i, l;
		mod || rethrow(Error("No such module: " + name));
		requires = mod["r"];
		script = mod["s"];
			
		// Create one-shot for table entry
		trigger = oneShotAsyncRaw();
		referencedModules[name] = addHandler = trigger["add"];
			
		// Make the one-shot call the callback
		isFunction(onDone) && addHandler(onDone);
			
		// When requires are finished
		allDone = pendingCallAsyncRaw(
			script === null ?
				trigger :
				function() { addScript(script, trigger); }
		);

		// For each requirement
		l = requires.length;
		Traverse {
			// Uses the async version on purpose
			requestModule(requires[i], allDone());
		}
		
		// If no requirements were given, this call releases allDone
		spawnApply(allDone, trueList);
	}
	
	requestModule["getModuleListing"] = getModule;
	requestModule["load"] = getLoadFunction();
	return requestModule;
}

function createPlaceholderModuleLoader() {
	var moduleLoader = null,
		run = oneShotAsyncRaw(),
		add = run["add"],
		loader = null,
		callLoader,
		setLoader;
	setLoader = function(newLoader) {
		if(loader !== null)
			rethrow(Error("Illegal state"));
		loader = newLoader;
		if(loader !== null) {
			run();
			visibleLoader["ready"] = true;
		}
	};
	visibleLoader = function() {
		var t = this, a = arguments;
		add(function() {
			loader.apply(t,a);
		});
	};
	visibleLoader["ready"] = false;
	setLoader.moduleLoader = visibleLoader;
	visibleLoader["getModuleListing"] = function() {
		if(loader === null)
			rethrow(Error("Not available while loading"));
		return loader["getModuleListing"].apply(loader,arguments);
	};
	visibleLoader["load"] = getLoadFunction();
	return setLoader;
}

function createModuleLoader(a0, a1) {
	if(isFunction(a0)) {
		return createModuleLoaderFromFunction(a0);
	}
	else if(isObject(a0)) {
		return createModuleLoaderFromFunction(function(key) {
			return a0[STR(key)];
		});
	}
	else if(isString(a0) && isFunction(a1)) {
		// In this scenario, the function is returned immediately, but no
		// requests are honored until after the script at a0 is loaded. When a0
		// is finished, a1 is called to return the function or object that
		// would have been passed to this function in the first place.
		var setLoader = createPlaceholderModuleLoader(),
			moduleLoader = setLoader.moduleLoader;
		addScript(a0, function() {
			setLoader(createModuleLoader(a1()));
		});
		return moduleLoader;
	}
	else {
		rethrow(TypeError());
	}
}

createModuleLoader["load"] = getLoadFunction();

