
#include "general.h"

var moduleTable = {};

function requestModule() {
	LOG("Module requested, args: ", arguments);
	spawnApply(requestModuleNonAsync, arguments);
}

// Calls to this function from other frames need to be serialized.
function requestModuleNonAsync(name, callWhenDone) {
	LOG("Request running non-async");
	if(owns(moduleTable, name)) {
		// It's already been referenced; just wait to be called.
		isFunction(callWhenDone) && moduleTable[name](callWhenDone);
		return;
	}
	
	// It hasn't yet been referenced. Load its requirements.
	var mod = getModuleSpec(name),
		addHandler, trigger, // For one-shot used for dependent calls
		pend, // For triggering one-shot once requires and script are done loading
		requires, script,
		i, l;
	
	mod || rethrow(Error("No such module: " + name));
	requires = mod["r"];
	script = mod["s"];
	
	LOG("Script location: ", script);
	LOG("Prereq: ", requires);
	
	// Generate the one-shot and use it to handle the table entry
	trigger = oneShotAsyncRaw();
	moduleTable[name] = addHandler = trigger["add"];
	
	// Make the one-shot call the callback
	isFunction(callWhenDone) && addHandler(callWhenDone);
	
	// What happens when requires are finished
	pend = pendingCallAsyncRaw(
		isSomething(script) ?
			function() {
				LOG("Pending script call starting: ", script);
				loadScriptThenSpawnRaw(script, trigger);
			} :
			trigger
	);
	
	// For each requirement
	if(isListLike(requires)) {
		l = requires.length;
		Traverse {
			// Uses the async version on purpose
			requestModule(requires[i], pend());
		}
	}
	
	// If no requirements were given, this call to pend will release the call
	spawnApply(pend, [true]);
}
