
#include "general.h"

// This file redefines several macros as functions so that JavaScript-only
// compilers may have an opportunity to use them for optimization. These
// variants also don't have the macro-related dangers, though they may be less
// efficient unless they end up optimized.

#define MAP(functionName, macroName, args...) \
	function functionName(args) { return macroName(args); }

MAP(owns, Owns, obj, key);
MAP(getObjectString, GetObjectString, obj);

#define OBJECT_TYPE_IS(type, value) \
	(getObjectString(value) === __OBJECT_TYPE_PREFIX + (#type) + __OBJECT_TYPE_SUFFIX)

MAP(isUndef, IsUndef, value);
MAP(isDef, IsDef, value);
MAP(isSomething, IsSomething, value);
MAP(isNothing, IsNothing, value);
MAP(isInt, IsInt, value);
MAP(isUInt, IsUInt, value);

function isArray(value) {
	return OBJECT_TYPE_IS(Array, value);
}
MAP(isObject, IsObject, value);
MAP(isString, IsString, value);
MAP(isNumber, IsNumber, value);
MAP(isBoolean, IsBoolean, value);
function isRegExp(value) {
	return OBJECT_TYPE_IS(RegExp, value);
}
function isFunction(value) {
	return OBJECT_TYPE_IS(Function, value);
}

function isListLike(value) {
	return isObject(value) && isUInt(value.length);
}

// Other peripherally related functions

// Allows a throw to be used as an expression
function rethrow(value) {
	throw value;
}


#define REQTYPE(capType) \
	function require ## capType(value) {\
		is ## capType(value) || rethrow(TypeError());\
	}

REQTYPE(Undef);
REQTYPE(Def);
REQTYPE(Something);
REQTYPE(Nothing);
REQTYPE(Int);
REQTYPE(UInt);
REQTYPE(Array);
REQTYPE(Object);
REQTYPE(String);
REQTYPE(Number);
REQTYPE(Boolean);
REQTYPE(RegExp);
REQTYPE(Function);
REQTYPE(ListLike);
